#include <winsock.h>
#include <malloc.h>
#include <process.h>
#include <winsock.h>

#include "Globals.h"
#include "BitmapConverter.h"

void	*user_io_ptr;
struct	image_info_t image_infos[ROWS * COLS];

int BitmapConverter::getBadTile()
{
	int		tile;

	if (_badTiles.size() > 0) {
		tile = _badTiles.front();
		_badTiles.pop();
		return tile;
	}

	return -1;
}

void BitmapConverter::sendImage(char *bitmap, int len, 
									int width, int height, 
									int tile, int socket)
{
	image_infos[tile].tile = tile;
	image_infos[tile].bitmap = bitmap;
	image_infos[tile].len = len;
	image_infos[tile].width = width;
	image_infos[tile].height = height;
	image_infos[tile].socket = socket;
	image_infos[tile].size = 0;

	convert_to_png(&image_infos[tile]);
	if (image_infos[tile].size < 100) {
			//XXX: hack to prevent black tiles from staying on the screen
		_badTiles.push(tile);
	}
} 	

void socket_write_data(png_structp png_ptr,
       png_bytep data, png_size_t length)
{
	struct		image_info_t *image_info;

	image_info = (struct image_info_t *) png_ptr->io_ptr;
	::send(image_info->socket, (char *) data, length, 0);
	image_info->size += length;
}

void socket_flush_data(png_structp png_ptr)
{
}

void png_begin(struct image_info_t *image_info)
{
	image_info->png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,NULL, NULL,
		NULL);

	image_info->info_ptr = png_create_info_struct(image_info->png_ptr);

	setjmp(image_info->png_ptr->jmpbuf);

	png_set_write_fn(image_info->png_ptr, (void *)user_io_ptr, socket_write_data, 
		 socket_flush_data);
}

void png_end(struct image_info_t *image_info)
{
	png_destroy_write_struct(&(image_info->png_ptr), &(image_info->info_ptr));
	shutdown(image_info->socket, 2);
	closesocket(image_info->socket);
}

void convert_to_png(void *param)
{
	struct      png_color_8_struct sig_bit;
	int			k=0;
	png_bytep	*row_pointers;
	int			tile_width, tile_height;
	int			tile_row, tile_col;
	int			scan_len;
	unsigned char * tile_start;
	char		*bitmap;
	int			tile, len, width, height, socket;
	struct		image_info_t *image_info;

	image_info = (struct image_info_t *) param;

	bitmap = image_info->bitmap;
	tile = image_info->tile;
	len = image_info->len;
	width = image_info->width;
	height = image_info->height;
	socket = image_info->socket;


	tile_row = tile / COLS;
	tile_col = tile % COLS;

	if (tile_col < COLS - 1) {
		tile_width = width / COLS;
	} else {
		tile_width = width - ((COLS - 1) * (width / COLS));
	}

	if (tile_row < ROWS - 1) {
		tile_height = height / ROWS;
	} else {
		tile_height = height - ((ROWS - 1) * (height / ROWS));
	}

	scan_len = width * BYTES_PER_PIXEL;
	row_pointers = (png_bytep*) _alloca(sizeof(png_bytep) * tile_height);
	tile_start = (unsigned char *) (bitmap) +  
			tile_row * (height / ROWS) * scan_len +
			(width / COLS) * tile_col * BYTES_PER_PIXEL;


	for (int i = 0; i < tile_height; i++) {
		row_pointers[i] = tile_start + i * scan_len;
	}

	png_begin(image_info);
	image_info->png_ptr->io_ptr = (void *) image_info;


	png_set_IHDR(image_info->png_ptr, image_info->info_ptr, tile_width, tile_height, 8,
		PNG_COLOR_TYPE_RGB,
		PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
		PNG_FILTER_TYPE_BASE);

	sig_bit.gray = 0;
	sig_bit.red = 8;
	sig_bit.green = 8;
	sig_bit.blue = 8;
	sig_bit.alpha = 0; 

	png_set_sBIT(image_info->png_ptr, image_info->info_ptr, &sig_bit);

	png_write_info(image_info->png_ptr, image_info->info_ptr);

	// flip BGR pixels to RGB 
	png_set_bgr(image_info->png_ptr);

	png_write_image(image_info->png_ptr, row_pointers);

	png_end(image_info);
}
