#include <iostream>

#include "ScreenScraper.h"
#include "Globals.h"
#include "GenUtils.h"

#define UPDATE_INTERVAL		100	//millisecs

ScreenScraper::ScreenScraper()
{
	int					ret;
	string				app = "\"";
	
	app += GenUtils::getWebRoot();
	app += "winmine.exe.lnk\"";

	_frameBuffer = 0;
	_frameBufferPrev = 0;
	_window = 0;
	_stopUpdates = false;
	_stopEvent = CreateEvent(NULL, FALSE, TRUE, NULL);

	ret = system(app.c_str());
	if (ret != 0) {
		cout << "Could not find " << app.c_str();
		exit(1);
	}
}

ScreenScraper::~ScreenScraper()
{
	restoreContext();
}

void ScreenScraper::attach(string program)
{
	_window = ::FindWindow(0, program.c_str());
	if (_window == 0) {
		cout << "Could not find app window";
		exit(1);
	}

	::SetWindowPos(_window, NULL, 0, 0, 508, 370, SWP_NOMOVE | SWP_NOZORDER);
	prepareContext();
	captureScreen();
}

void ScreenScraper::startUpdates(StreamSocket *socket)
{
	if (_window == 0) {
		attach("Minesweeper");
	}

	_stopUpdates = true;
	WaitForSingleObject(_stopEvent, INFINITE);
	_stopUpdates = false;

	_clientSocket = socket;
	start();	// Start the thread and call run().
}

void ScreenScraper::run()
{
	while (_stopUpdates == false) {
		getUpdates();
		if (_updateList.empty() == false) {
			_clientSocket->send(_updateList.c_str(), _updateList.length());
		}
		sleep(UPDATE_INTERVAL);
	}
	_clientSocket->close();
	SetEvent(_stopEvent);
}

void ScreenScraper::getUpdates() 
{
	int		badTile;
	char	tileString[16];

	memcpy(_frameBufferPrev, _frameBuffer, _bufferLen);
	captureScreen();
	_updateList.erase();

	badTile = _converter.getBadTile();
	while (badTile != -1) {
		sprintf(tileString, "%d", badTile);
		_updateList += tileString;
		_updateList += ' ';
		badTile = _converter.getBadTile();
	} 

	for (int i = 0; i < ROWS * COLS; i++) {
		if (detectChange(i) == true) {
			sprintf(tileString, "%d", i);
			_updateList += tileString;
			_updateList += ' ';
		}
	}
}

bool ScreenScraper::detectChange(int tile)
{
	int			tile_width, tile_height;
	int			tile_row, tile_col;
	int			scan_len;
	int			tile_start;

	tile_row = tile / COLS;
	tile_col = tile % COLS;

	if (tile_col < COLS - 1) {
		tile_width = _width / COLS;
	} else {
		tile_width = _width - ((COLS - 1) * (_width / COLS));
	}

	if (tile_row < ROWS - 1) {
		tile_height = _height / ROWS;
	} else {
		tile_height = _height - ((ROWS - 1) * (_height / ROWS));
	}

	scan_len = _width * BYTES_PER_PIXEL;
	tile_start = tile_row * (_height / ROWS) * scan_len +
			(_width / COLS) * tile_col * BYTES_PER_PIXEL;

	for (int i = 0; i < tile_height; i++) {
		for (int j = 0; j < tile_width * BYTES_PER_PIXEL; j++) {
			int pixel;

			pixel = tile_start + i * scan_len + j;
			if (_frameBuffer[pixel] != _frameBufferPrev[pixel]) {
				return true;
			}
		}
	}

	return false;
}

void ScreenScraper::sendImage(int tile, int socket)
{
	if (_window == 0) {
		attach("Minesweeper");
	}

	_converter.sendImage(_frameBuffer, _bufferLen, _width, _height, tile, socket);
}

BOOL ScreenScraper::prepareContext() {
	RECT rc;

	if (_window == 0) {
		return FALSE;
	}

	_scrContext		= ::GetWindowDC(_window);

	
	if (!_scrContext)
		return FALSE;
	_memContext 	= ::CreateCompatibleDC(_scrContext);
	
	if (!_memContext) {
		::ReleaseDC(_window, _scrContext);
		return FALSE;
	}

	int nRet = ::GetWindowRect(_window, &rc);
	if(nRet == 0) {
		return FALSE;
	}
	_width 	= rc.right - rc.left;
	_height 	= rc.bottom - rc.top;

	_height -= TITLE_HEIGHT;

//XXX: libpng crashes without this.
	_width += _width % 2;
	_height += _height % 2;
//XXX

	_memBitmap = CreateCompatibleBitmap(_scrContext, _width, _height);
	_scrBitmap = (HBITMAP) SelectObject(_memContext, _memBitmap);

	_bufferLen = _width * _height * BYTES_PER_PIXEL;
	_frameBuffer = new char[_bufferLen];
	_frameBufferPrev = new char[_bufferLen];
	memset(_frameBuffer, _bufferLen, 0);
	memset(_frameBufferPrev, _bufferLen, 0);

	_bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	_bmi.bmiHeader.biWidth = _width;
	_bmi.bmiHeader.biHeight = -1 * _height;
	_bmi.bmiHeader.biPlanes = 1;
	_bmi.bmiHeader.biBitCount = BYTES_PER_PIXEL * 8;
	_bmi.bmiHeader.biCompression = BI_RGB;
	_bmi.bmiHeader.biSizeImage = 0;

	return TRUE;
}

void ScreenScraper::restoreContext() {
	SelectObject (_memContext, _scrBitmap);
	::ReleaseDC(_window, _scrContext);
	DeleteDC (_memContext);
	if(_frameBuffer)
		delete[] _frameBuffer;
	if(_frameBufferPrev)
		delete[] _frameBufferPrev;
}

inline void ScreenScraper::captureScreen() {
	BitBlt(_memContext, 0, 0, _width, _height, _scrContext, 0, TITLE_HEIGHT, SRCCOPY);	
	int lines = GetDIBits(_memContext, _memBitmap, 0, _height, _frameBuffer, &_bmi, DIB_RGB_COLORS);
}




