#include <string.h>
#include <string>
#include <fstream>
#include <iostream>
#include "GenUtils.h"
#include "ScreenServer.h"

#define	SERVER_PORT		80

int main(int argc, char* argv[])
{
	ScreenServer	server;

	server.start();
	return 0;
}

ScreenServer::ScreenServer()
{
	BOOL	ret;

	_clientSocket = 0;
	_socket.create();
	ret = _socket.bind(SERVER_PORT);
	if (ret == FALSE) {
		cout << "Port " << SERVER_PORT << " is taken. Exiting...";
		exit(1);
	}
	_socket.listen();
	cout << "Server ready..." << endl;
}

void ScreenServer::start()
{
	while(true) {
		_clientSocket = _socket.accept();
		if (_clientSocket == 0) {
			cout << "Socket error. Exiting...";
			exit(1);
		}
		runSession();
	}
}

void ScreenServer::runSession()
{
	string	request;

	readRequest(request);
	if (request == "") {
		return;
	}
	
	if (request == "updates?") {
		sendUpdates();
	} else if (request.find("image?") == 0) {
		int	tile;
		tile = GenUtils::getParam(request, 0);
		sendImage(tile);
	} else if (request.find("event?") == 0) {
		handleEvent(request);
	} else {
		if (request == "index.html") {
			cout << "Request from " << _clientSocket->peer().c_str() << endl;
		}
		sendFile(request);
	}
}

void ScreenServer::readRequest(string& request)
{
	char	inbuf[1024] = {0};
	int		ret;

	ret = _clientSocket->receiveChunk(inbuf, sizeof(inbuf));
	request = inbuf;
	if (request == "") {
		return;
	}

	request = request.substr(5, request.size());
	request = request.substr(0, request.find(" ", 0));
	if (request.empty() == true) {
		request = "index.html";
	}
}

void ScreenServer::sendHeader(string& contentType)
{
	char	header[256] = {0};

	sprintf(header, "HTTP/1.1 200 OK\r\n"
					"Cache-Control: no-cache\r\n"
					"Content-Type: %s\r\n"
					"Connection: Keep-Alive\r\n\r\n", contentType.c_str());
	_clientSocket->send(header, strlen(header));
}

void ScreenServer::sendUpdates()
{
	string	contentType = "text/html";

	sendHeader(contentType);
	_scraper.startUpdates(_clientSocket);
}

void ScreenServer::sendImage(int tile)
{
	string	contentType;

	if (tile == ROWS * COLS) {
			//XXX: hack for event handling.
		_clientSocket->close();
	} else {
		contentType = "image/png";
		sendHeader(contentType);
		_scraper.sendImage(tile, _clientSocket->getSocket());
	}
}

void ScreenServer::sendFile(string& file)
{
	char		header[256] = {0};
	int			fileLen;
	char		*buf;
	string		path, ext, contentType;
	
	path = GenUtils::getWebRoot();
	path += file;

	ifstream	infile(path.c_str(), ios::in | ios::binary);
	
	fileLen = GenUtils::getSize(infile);
	if (fileLen == -1) {
		sendError();
		_clientSocket->close();
		return;
	}

	ext = GenUtils::getExt(path);
	if (ext == "jpg") {
		contentType = "image/jpeg";
	} else if (ext == "png") {
		contentType = "image/png";
	} else {
		contentType = "text/html";
	}

	buf = new char[fileLen];
	infile.read(buf, fileLen);

	sendHeader(contentType);
	_clientSocket->send(buf, fileLen);
	_clientSocket->close();
	
	delete[] buf;
}

void ScreenServer::handleEvent(string& request)
{
	int					param1, param2;
	EventHandler::Type	event;
	
	_clientSocket->close();
	event = (EventHandler::Type) GenUtils::getParam(request, 0);

	param1 = GenUtils::getParam(request, 1);
	param2 = GenUtils::getParam(request, 2);
	_eventHandler.processEvent(_scraper.getWindow(), event, param1, param2);
}

void ScreenServer::sendError()
{
	//XXX
}


