#include <winsock2.h>
#include "streamsocket.h"

int StreamSocket::_init = StreamSocket::init();

StreamSocket::StreamSocket() : _socket(0) 
{
}

StreamSocket::StreamSocket(char *host, int port) : _socket(0) 
{
}

StreamSocket::~StreamSocket() 
{
	if(valid()) {
		close();
	}
}

int StreamSocket::init() 
{
	WORD	version;
	WSADATA wsad;
	int		ret;

	version = MAKEWORD(2, 0);
	ret = WSAStartup(version, &wsad);
	if(ret != 0) {
		return FALSE;
	}
	return TRUE;
}

int StreamSocket::valid() 
{
	if(!_init) {
		return FALSE;
	}
	if(!_socket) {
		return FALSE;
	}
	return TRUE;
}

int StreamSocket::create() 
{
	if(!_init) {
		_init = init();
	}

	if((_socket = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		return FALSE;
	}
	return TRUE;	
}

int StreamSocket::bind(int port) 
{
	if(!valid())
		return FALSE;

	struct sockaddr_in addr;

	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = INADDR_ANY;

	if(::bind(_socket, (struct sockaddr *)&addr, sizeof(addr)) == SOCKET_ERROR) {
		return FALSE;
	}
	return TRUE;
}
	
int StreamSocket::connect(char *host, int port) 
{
	
	if(!valid())
		return FALSE;

	struct sockaddr_in addr;
	memset(&addr, 0, sizeof(addr));

	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(host);

	if(addr.sin_addr.s_addr == INADDR_NONE) {
		struct hostent *st_hostent;
		st_hostent = gethostbyname(host);
		if (st_hostent != NULL) {
			if (st_hostent->h_addr == NULL) {
				return FALSE;
			}
			addr.sin_addr.s_addr = ((struct in_addr *)st_hostent->h_addr)->s_addr;
		} else {
			return FALSE;
		}
	}
	
	addr.sin_port = htons(port);

	if (::connect(_socket, (struct sockaddr *)&addr, sizeof(addr)) == 0)
		return TRUE;

	return FALSE;
}

int StreamSocket::send(const char* buf, int len) 
{
	if(!valid())
		return SOCKET_ERROR;

	int ret = ::send(_socket, buf, len, 0);
	if (ret == -1) 
		int err = WSAGetLastError();
	return ret;
}

int StreamSocket::listen() 
{
	if(!valid())
		return FALSE;

	if (::listen(_socket, 5) == SOCKET_ERROR) {
		return FALSE;
	}
	return TRUE;
}

StreamSocket* StreamSocket::accept() 
{
	if(!valid())
		return NULL;

	StreamSocket *new_socketet;
	int new_socketet_id;

	if((new_socketet_id = ::accept(_socket, 0, 0)) == INVALID_SOCKET) {
		return NULL;
	}

	new_socketet = new StreamSocket();
	new_socketet->_socket = new_socketet_id;

	return new_socketet;
}

int StreamSocket::receive(char* buf, int len) 
{
	int left = len, ret;

	if(!valid())
		return SOCKET_ERROR;

	while(left) {
		ret = recv(_socket, buf, left, 0);
		left -= ret;
		buf += ret;
		if(ret == SOCKET_ERROR) {
			break;
		}
	}
	return (len - left);
}

int StreamSocket::receiveChunk(char* buf, int len) 
{
	int ret;

	if(!valid())
		return SOCKET_ERROR;

	ret = recv(_socket, buf, len, 0);

	return ret;
}

string StreamSocket::peer() 
{
	string				peer;
	struct sockaddr_in	sockinfo;
	struct in_addr		address;
	int					sockinfosize = sizeof(sockinfo);

	getpeername(_socket, (struct sockaddr *)&sockinfo, &sockinfosize);
	memcpy(&address, &sockinfo.sin_addr, sizeof(address));

	peer = inet_ntoa(address);
	if (peer.empty() == true) {
		peer = "unknown";
	}

	return peer;
}

int StreamSocket::close() 
{
	if(!valid())
		return SOCKET_ERROR;
	shutdown(_socket, SD_BOTH);
	int ret = closesocket(_socket);
	_socket = 0;

	return ret;
}